import 'package:apollopay/src/api/config.dart';
import 'package:apollopay/src/api/services/cards_service.dart';
import 'package:apollopay/src/api/services/customers_service.dart';
import 'package:apollopay/src/api/services/invoices_service.dart';
import 'package:apollopay/src/api/services/prices_service.dart';
import 'package:apollopay/src/api/services/products_service.dart';
import 'package:apollopay/src/api/services/rates_service.dart';
import 'package:apollopay/src/api/services/refunds_service.dart';
import 'package:apollopay/src/api/services/reports_service.dart';
import 'package:apollopay/src/api/services/transactions_service.dart';
import 'package:apollopay/src/api/services/wallets_service.dart';

class Apollopay {
  final String apiUrl;
  final String applicationToken;
  final String apiVersion;
  final String clientPublicKey;
  final String clientPrivateKey;

  late final CardsService cardsService;
  late final CustomersService customersService;
  late final InvoicesService invoicesService;
  late final PricesService pricesService;
  late final ProductsService productsService;
  late final RatesService ratesService;
  late final RefundsService refundsService;
  late final ReportsService reportsService;
  late final TransactionsService transactionsService;
  late final WalletsService walletsService;

  Apollopay(
      {required this.applicationToken,
      required this.apiUrl,
      required this.apiVersion,
      required this.clientPublicKey,
      required this.clientPrivateKey}) {
    Config.init(apiUrl, applicationToken, apiVersion, clientPublicKey,
        clientPrivateKey);
    cardsService =
        CardsService(endpoint: '/pay/cards/', baseUrl: Config.instance.baseUrl);
    customersService = CustomersService(
        endpoint: '/pay/customers/', baseUrl: Config.instance.baseUrl);
    invoicesService = InvoicesService(
        endpoint: '/pay/invoices/', baseUrl: Config.instance.baseUrl);
    pricesService = PricesService(
        endpoint: '/pay/prices/', baseUrl: Config.instance.baseUrl);
    productsService = ProductsService(
        endpoint: '/pay/products/', baseUrl: Config.instance.baseUrl);
    ratesService = RatesService(
        endpoint: '/crypto/rates/', baseUrl: Config.instance.baseUrl);
    refundsService = RefundsService(
        endpoint: '/pay/refunds/', baseUrl: Config.instance.baseUrl);
    reportsService = ReportsService(
        endpoint: '/pay/reports/', baseUrl: Config.instance.baseUrl);
    transactionsService = TransactionsService(
        endpoint: '/pay/charges/', baseUrl: Config.instance.baseUrl);
    walletsService = WalletsService(
        endpoint: '/crypto/wallets/', baseUrl: Config.instance.baseUrl);
  }
}
