import 'dart:convert';

import 'package:apollopay/src/api/entities/entity_entity.dart';

class Address extends Entity {
  String? number;
  String? street;
  String? apt;
  String? city;
  String? state;
  String? country;
  String? zipCode;

  Address({
    this.number,
    this.street,
    this.apt,
    this.city,
    this.state,
    this.country,
    this.zipCode,
  });

  Address copyWith({
    String? number,
    String? street,
    String? apt,
    String? city,
    String? state,
    String? country,
    String? zipCode,
  }) {
    return Address(
      number: number ?? this.number,
      street: street ?? this.street,
      apt: apt ?? this.apt,
      city: city ?? this.city,
      state: state ?? this.state,
      country: country ?? this.country,
      zipCode: zipCode ?? this.zipCode,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'number': number,
      'street': street,
      'apt': apt,
      'city': city,
      'state': state,
      'country': country,
      'zip': zipCode,
    };
  }

  factory Address.fromMap(Map<String, dynamic> map) {
    return Address(
      number: map['number'],
      street: map['street'],
      apt: map['apt'],
      city: map['city'],
      state: map['state'],
      country: map['country'],
      zipCode: map['zip'],
    );
  }

  factory Address.fromJson(String source) =>
      Address.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Address(number: $number, street: $street, apt: $apt, city: $city, province: $state, zipCode: $zipCode)';
  }

  @override
  bool operator ==(Object? other) {
    if (identical(this, other)) return true;

    return other is Address &&
        other.number == number &&
        other.street == street &&
        other.apt == apt &&
        other.city == city &&
        other.state == state &&
        other.country == country &&
        other.zipCode == zipCode;
  }

  @override
  int get hashCode {
    return number.hashCode ^
        street.hashCode ^
        apt.hashCode ^
        city.hashCode ^
        state.hashCode ^
        country.hashCode ^
        zipCode.hashCode;
  }
}
