import 'dart:convert';

import 'package:apollopay/src/api/entities/entity_entity.dart';
import 'package:mongo_dart/mongo_dart.dart';

class Client extends Entity {
  String? id;
  String? company;
  dynamic credentials;
  dynamic keys;

  Client(
      {this.id, this.company, required this.credentials, required this.keys});

  @override
  Client copyWith(
      {String? id, String? company, dynamic credentials, dynamic keys}) {
    return Client(
        id: id ?? this.id,
        company: company ?? this.company,
        credentials: credentials ?? this.credentials,
        keys: keys ?? this.keys);
  }

  @override
  Map<String, dynamic> toMap() {
    return {
      '_id': id,
      'company': company,
      'credentials': credentials,
      'keys': keys
    };
  }

  factory Client.fromMap(Map<String, dynamic> map) {
    return Client(
        id: map['_id'] is ObjectId
            ? map['_id'].toHexString()
            : map['_id'].toString(),
        company: map['company'],
        credentials: map['credentials'],
        keys: map['keys']);
  }

  factory Client.fromJson(String source) => Client.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Client(id: $id, company: $company, credentials: $credentials, keys: $keys)';
  }

  @override
  bool operator ==(Object? other) {
    if (identical(this, other)) return true;

    return other is Client &&
        other.id == id &&
        other.company == company &&
        other.credentials == credentials &&
        other.keys == keys;
  }

  @override
  int get hashCode {
    return id.hashCode ^
        company.hashCode ^
        credentials.hashCode ^
        keys.hashCode;
  }
}
