import 'dart:convert';
import 'package:apollopay/src/api/entities/crypto/currency_entity.dart';
import 'package:apollopay/src/api/entities/entity_entity.dart';
import 'package:mongo_dart/mongo_dart.dart';

class Wallet extends Entity {
  String? id;
  Currency currency;
  String? token;
  String address;
  String publicKey;
  String privateKey;
  String name;
  String userId;
  BigInt? balance;

  Wallet(
      {required this.id,
      required this.currency,
      required this.address,
      required this.publicKey,
      required this.privateKey,
      required this.name,
      required this.userId,
      this.token,
      this.balance});

  @override
  Wallet copyWith(
      {String? id,
      Currency? currency,
      String? token,
      String? address,
      String? publicKey,
      String? privateKey,
      String? name,
      String? userId,
      BigInt? balance}) {
    return Wallet(
      id: id ?? this.id,
      currency: currency ?? this.currency,
      token: token ?? this.token,
      address: address ?? this.address,
      publicKey: publicKey ?? this.publicKey,
      privateKey: privateKey ?? this.privateKey,
      name: name ?? this.name,
      userId: userId ?? this.userId,
      balance: balance ?? this.balance,
    );
  }

  @override
  Map<String, dynamic> toMap() {
    return {
      '_id': id,
      'currency': currency.index,
      'token': token?.toString(),
      'address': address,
      'publicKey': publicKey,
      'privateKey': privateKey,
      'name': name,
      'userId': userId,
      'balance': balance?.toString()
    };
  }

  factory Wallet.fromMap(Map<String, dynamic> map) {
    return Wallet(
      id: map['_id'] is ObjectId
          ? map['_id'].toHexString()
          : map['_id'].toString(),
      currency: (map['currency'] != null)
          ? Currency.values[int.parse(map['currency'].toString())]
          : Currency.none,
      token: map['token']?.toString(),
      address: map['address'] ?? '',
      publicKey: map['publicKey'] ?? '',
      privateKey: map['privateKey'] ?? '',
      name: map['name'] ?? '',
      userId: map['userId'] ?? '',
      balance: BigInt.tryParse(map['balance'].toString()),
    );
  }

  factory Wallet.fromJson(String source) => Wallet.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Wallet(id: $id, currency: $currency, publicKey: $publicKey, privateKey: $privateKey, name: $name, userId: $userId)';
  }

  @override
  bool operator ==(Object? other) {
    if (identical(this, other)) return true;

    return other is Wallet &&
        other.id == id &&
        other.currency == currency &&
        other.publicKey == publicKey &&
        other.privateKey == privateKey &&
        other.name == name &&
        other.userId == userId;
  }

  @override
  int get hashCode {
    return id.hashCode ^
        currency.hashCode ^
        publicKey.hashCode ^
        privateKey.hashCode ^
        name.hashCode ^
        userId.hashCode;
  }
}
