// ignore_for_file: constant_identifier_names

import 'dart:convert';

import 'package:apollopay/src/api/entities/address_entity.dart';
import 'package:apollopay/src/api/entities/entity_entity.dart';
import 'package:mongo_dart/mongo_dart.dart';

enum CardBrand {
  none,
  VISA,
  VISA_DEBIT,
  ELECTRON,
  MASTERCARD,
  DEBIT_MASTERCARD,
  MAESTRO,
  LASER,
  AMEX,
  DINERS,
  JCB,
  DISCOVER,
  CUP_SECUREPAY,
}

class Card extends Entity {
  CardBrand brand;
  String? client;
  String? expiration;
  String? token;
  Address? address;
  String? customer;
  String? cardholderName;
  String? currency;
  String? number;
  String? cvv;
  String? lastFour;
  String? providerReference;
  bool? active;

  Card(
      {this.expiration,
      this.client,
      this.token,
      this.address,
      this.customer,
      this.cardholderName,
      this.currency,
      this.number,
      this.cvv,
      this.lastFour,
      this.providerReference,
      this.brand = CardBrand.none,
      this.active});

  @override
  Card copyWith({
    CardBrand? brand,
    String? client,
    String? expiration,
    String? token,
    Address? address,
    String? customer,
    String? cardholderName,
    String? currency,
    String? number,
    String? cvv,
    String? lastFour,
    String? providerReference,
    bool? active,
  }) {
    return Card(
        brand: brand ?? this.brand,
        client: client ?? this.client,
        expiration: expiration ?? this.expiration,
        token: token ?? this.token,
        address: address ?? this.address,
        customer: customer ?? this.customer,
        cardholderName: cardholderName ?? this.cardholderName,
        currency: currency ?? this.currency,
        number: number ?? this.number,
        cvv: cvv ?? this.cvv,
        lastFour: lastFour ?? this.lastFour,
        providerReference: providerReference ?? this.providerReference,
        active: active ?? this.active);
  }

  @override
  Map<String, dynamic> toMap({bool databaseFormat = false}) {
    return {
      if (number == null) 'token': token,
      'expiration': expiration,
      'address': address?.toMap(),
      'customer': customer,
      'cardholder_name': cardholderName,
      if (currency != null) 'currency': currency,
      if (token == null) 'number': number,
      if (token == null) 'cvv': cvv,
      if (databaseFormat) 'client': client,
      'last_four': lastFour,
      'brand': brand.index,
      if (databaseFormat) 'provider_reference': providerReference,
      'active': active
    };
  }

  factory Card.fromMap(Map<String, dynamic> map) {
    assert(map['token'] != null || map['number'] != null,
        "CardError -> token or number must be set.");
    assert(map['expiration'] != null, "CardError -> expiration must be set.");
    assert(map['cardholder_name'] != null,
        "CardError -> cardholder_name name must be set.");
    assert(
        map['token'] != null || (map['number'] != null && map['cvv'] != null),
        "CardError -> cvv must be specified for a new card.");

    return Card(
        client: map['client']?.toString() ?? '',
        expiration: map['expiration']?.toString().padLeft(4, '0'),
        token: map['token'],
        address:
            map['address'] != null ? Address.fromMap(map['address']) : null,
        customer: map['customer'],
        cardholderName: map['cardholder_name'],
        currency: map['currency'],
        number:
            map['number']?.toString().replaceAll(' ', '').replaceAll('-', ''),
        cvv: map['cvv'],
        lastFour: map['last_four'],
        providerReference: map['provider_reference'],
        brand: (map['brand'] != null)
            ? CardBrand
                .values[int.parse(map['brand'].toString().replaceAll('.0', ''))]
            : CardBrand.none,
        active: map['active'] ?? true);
  }

  factory Card.fromJson(String source) => Card.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Card(expiration: $expiration, id: $token, address: $address, customer: $customer, cardholderName: $cardholderName, currency: $currency, number: $number, cvv: $cvv, lastFour: $lastFour)';
  }

  @override
  bool operator ==(Object? other) {
    if (identical(this, other)) return true;

    return other is Card &&
        other.expiration == expiration &&
        other.token == token &&
        other.address == address &&
        other.customer == customer &&
        other.cardholderName == cardholderName &&
        other.currency == currency &&
        other.number == number &&
        other.cvv == cvv &&
        other.lastFour == lastFour;
  }

  @override
  int get hashCode {
    return expiration.hashCode ^
        token.hashCode ^
        address.hashCode ^
        customer.hashCode ^
        cardholderName.hashCode ^
        currency.hashCode ^
        number.hashCode ^
        cvv.hashCode ^
        lastFour.hashCode;
  }
}
