import 'dart:convert';

import 'package:apollopay/src/api/entities/address_entity.dart';
import 'package:apollopay/src/api/entities/entity_entity.dart';
import 'package:mongo_dart/mongo_dart.dart';

class Customer extends Entity {
  String? id;
  String? token;
  String? client;
  String? firstName;
  String? lastName;
  String? company;
  String? email;
  Address? address;
  Map<String, String>? metadata;
  DateTime? creationDate;

  String get displayName {
    if (company != null) {
      return company!;
    }

    return '$firstName $lastName';
  }

  Customer(
      {this.id,
      this.token,
      required this.client,
      this.firstName,
      this.lastName,
      this.company,
      this.email,
      this.address,
      this.metadata,
      this.creationDate});

  Map<String, dynamic> toMap({bool databaseFormat = false}) {
    return {
      // if (databaseFormat) '_id': id,
      if (databaseFormat) 'client': client,
      'token': token,
      'first_name': firstName,
      'last_name': lastName,
      'company': company,
      'email': email,
      'address': address?.toMap(),
      'metadata': metadata,
      'creation_date': creationDate?.millisecondsSinceEpoch
    };
  }

  factory Customer.fromMap(Map<String, dynamic> map) {
    assert(map['first_name'] != null && map['last_name'] != null,
        'CustomerError -> first_name and last_name must be set.');

    return Customer(
        id: map['_id'] is ObjectId
            ? map['_id']?.toHexString()
            : map['_id']?.toString(),
        client: map['client'],
        token: map['token'],
        firstName: map['first_name'],
        lastName: map['last_name'],
        company: map['company'],
        email: map['email'],
        address:
            map['address'] != null ? Address.fromMap(map['address']) : null,
        metadata: (map['metadata'] != null) ? Map.from(map['metadata']) : {},
        creationDate: (map['creation_date'] != null)
            ? DateTime.fromMillisecondsSinceEpoch(
                int.parse(map['creation_date'].toString()))
            : DateTime.now());
  }

  factory Customer.fromJson(String source) =>
      Customer.fromMap(json.decode(source));

  @override
  String toString() {
    return '$firstName $lastName';
  }

  @override
  bool operator ==(Object? other) {
    if (identical(this, other)) return true;

    return other is Customer &&
        other.id == id &&
        other.client == client &&
        other.firstName == firstName &&
        other.lastName == lastName &&
        other.company == company &&
        other.email == email &&
        other.address == address;
  }

  @override
  int get hashCode {
    return id.hashCode ^
        client.hashCode ^
        firstName.hashCode ^
        lastName.hashCode ^
        company.hashCode ^
        email.hashCode ^
        address.hashCode;
  }

  Customer copyWith({
    String? firstName,
    String? lastName,
    String? company,
    String? email,
    Address? address,
    Map<String, String>? metadata,
  }) {
    return Customer(
      id: id,
      token: token,
      client: client,
      firstName: firstName ?? this.firstName,
      lastName: lastName ?? this.lastName,
      company: company ?? this.company,
      email: email ?? this.email,
      address: address ?? this.address,
      metadata: metadata ?? this.metadata,
    );
  }
}
