import 'dart:convert';

import 'package:apollopay/src/api/entities/entity_entity.dart';

class Tier extends Entity {
  int amount;
  int unitAmount;
  double? upperLimit;
  Tier({
    required this.amount,
    required this.unitAmount,
    this.upperLimit,
  });

  @override
  Tier copyWith({int? amount, int? unitAmount, double? upperLimit}) {
    return Tier(
        amount: amount ?? this.amount,
        unitAmount: unitAmount ?? this.unitAmount,
        upperLimit: upperLimit ?? this.upperLimit);
  }

  @override
  Map<String, dynamic> toMap() {
    return {
      'amount': amount,
      'unitAmount': unitAmount,
      'upperLimit': upperLimit,
    };
  }

  factory Tier.fromMap(Map<String, dynamic> map) {
    return Tier(
      amount: map['amount']?.toInt() ?? 0,
      unitAmount: map['unitAmount']?.toInt() ?? 0,
      upperLimit: map['upperLimit']?.toDouble(),
    );
  }

  factory Tier.fromJson(String source) => Tier.fromMap(json.decode(source));

  @override
  String toString() =>
      'Tier(amount: $amount, unitAmount: $unitAmount, upperLimit: $upperLimit)';

  @override
  bool operator ==(Object? other) {
    if (identical(this, other)) return true;

    return other is Tier &&
        other.amount == amount &&
        other.unitAmount == unitAmount &&
        other.upperLimit == upperLimit;
  }

  @override
  int get hashCode =>
      amount.hashCode ^ unitAmount.hashCode ^ upperLimit.hashCode;
}
