import 'dart:convert';

import 'package:apollopay/src/api/entities/entity_entity.dart';
import 'package:apollopay/src/api/entities/payment/pricing/price_entity.dart';
import 'package:mongo_dart/mongo_dart.dart';

class Product extends Entity {
  String? id;
  String? token;
  String? client;
  String? thumbnail;
  String? name;
  String? description;
  bool? active;
  String? unitLabel;
  Map<String, String>? metadata;
  List<String>? medias;
  DateTime? creationDate;
  bool archived;

  //Not in bd
  List<Price> prices;

  Product({
    this.id,
    this.token,
    required this.client,
    this.thumbnail,
    this.name,
    this.description,
    this.active,
    this.unitLabel,
    this.metadata,
    this.medias,
    this.creationDate,
    this.archived = false,
    this.prices = const [],
  });

  @override
  Product copyWith({
    String? thumbnail,
    String? name,
    String? description,
    bool? active,
    String? unitLabel,
    Map<String, String>? metadata,
    List<String>? medias,
    DateTime? creationDate,
    bool? archived,
    List<Price>? prices,
  }) {
    return Product(
      id: id,
      token: token,
      client: client,
      thumbnail: thumbnail ?? this.thumbnail,
      name: name ?? this.name,
      description: description ?? this.description,
      active: active ?? this.active,
      unitLabel: unitLabel ?? this.unitLabel,
      metadata: metadata ?? this.metadata,
      medias: medias ?? this.medias,
      creationDate: creationDate ?? this.creationDate,
      archived: this.archived,
      prices: prices ?? this.prices,
    );
  }

  @override
  Map<String, dynamic> toMap({bool databaseFormat = false}) {
    return {
      if (databaseFormat) 'client': client,
      'token': token,
      'thumbnail': thumbnail,
      'name': name,
      'description': description,
      'active': active,
      'unit_label': unitLabel,
      'metadata': metadata,
      'medias': medias,
      'creation_date': creationDate?.millisecondsSinceEpoch,
      'archived': archived,
      if (!databaseFormat) 'prices': prices
    };
  }

  factory Product.fromMap(Map<String, dynamic> map) {
    assert(map['name'] != null, 'ProductError -> name must be set.');

    List<Price> prices = map['prices'] != null
        ? List<Price>.from(map['prices'].map((e) => Price.fromMap(e)))
        : [];
    return Product(
        id: map['_id'] is ObjectId
            ? map['_id'].toHexString()
            : map['_id'].toString(),
        client: map['client'],
        token: map['token'],
        thumbnail: map['thumbnail'],
        name: map['name'],
        active: map['active'] ?? true,
        description: map['description'],
        unitLabel: map['unit_label'],
        metadata: (map['metadata'] != null) ? Map.from(map['metadata']) : {},
        medias: (map['medias'] != null)
            ? List.from(map['medias']).map((e) => e.toString()).toList()
            : [],
        creationDate: (map['creation_date'] != null)
            ? DateTime.fromMillisecondsSinceEpoch(
                int.parse(map['creation_date'].toString()))
            : DateTime.now(),
        archived: map['archived'] ?? false,
        prices: prices);
  }

  factory Product.fromJson(String source) =>
      Product.fromMap(json.decode(source));

  @override
  String toString() {
    return 'Product(id: $id, client: $client, thumbnail: $thumbnail name: $name, description: $description, active: $active, unitLabel: $unitLabel, metadata: $metadata, medias: $medias, creationDate: $creationDate)';
  }

  @override
  bool operator ==(Object? other) {
    if (identical(this, other)) return true;

    return other is Product &&
        other.id == id &&
        other.client == client &&
        other.thumbnail == thumbnail &&
        other.name == name &&
        other.active == active &&
        other.medias == medias &&
        other.description == description &&
        other.unitLabel == unitLabel &&
        other.metadata == metadata &&
        other.creationDate == creationDate;
  }

  @override
  int get hashCode {
    return id.hashCode ^
        client.hashCode ^
        thumbnail.hashCode ^
        name.hashCode ^
        description.hashCode ^
        medias.hashCode ^
        active.hashCode ^
        unitLabel.hashCode ^
        metadata.hashCode ^
        creationDate.hashCode;
  }
}
