import 'dart:convert';

import 'package:apollopay/src/api/entities/entity_entity.dart';

class SalesDailyReport extends Entity {
  DateTime? min;
  DateTime? max;
  SalesDailyReport({
    this.min,
    this.max,
  });

  @override
  SalesDailyReport copyWith({
    DateTime? min,
    DateTime? max,
  }) {
    return SalesDailyReport(
      min: min ?? this.min,
      max: max ?? this.max,
    );
  }

  @override
  Map<String, dynamic> toMap() {
    return {
      'min': min?.millisecondsSinceEpoch,
      'max': max?.millisecondsSinceEpoch,
    };
  }

  factory SalesDailyReport.fromMap(Map<String, dynamic> map) {
    return SalesDailyReport(
      min: map['min'] != null
          ? DateTime.fromMillisecondsSinceEpoch(map['min'])
          : null,
      max: map['max'] != null
          ? DateTime.fromMillisecondsSinceEpoch(map['max'])
          : null,
    );
  }

  Map<String, dynamic> toJson() => toMap();

  factory SalesDailyReport.fromJson(String source) =>
      SalesDailyReport.fromMap(json.decode(source));

  @override
  String toString() => 'SalesDailyReport(min: $min, max: $max)';

  @override
  bool operator ==(Object? other) {
    if (identical(this, other)) return true;

    return other is SalesDailyReport && other.min == min && other.max == max;
  }

  @override
  int get hashCode => min.hashCode ^ max.hashCode;
}
