import 'dart:convert';
import 'package:apollopay/src/api/config.dart';
import 'package:apollopay/src/api/entities/payment/card_entity.dart';
import 'package:apollopay/src/api/services/client_http_service.dart';
import 'package:http/http.dart' as http;

class CardsService extends ClientHttpService {
  CardsService({
    required String baseUrl,
    String endpoint = "/pay/cards/",
  }) : super(baseUrl: baseUrl, endpoint: endpoint);

  Future<Card> getCard(String cardToken) async {
    return await http.get(Uri.parse('$baseUrl$endpoint$cardToken'), headers: {
      'content-type': 'application/json',
      'x-auth-token': applicationToken ?? '',
      'x-auth-private': Config.instance.clientPrivateKey,
      'x-auth-public': Config.instance.clientPublicKey,
    }).then((response) async {
      switch (response.statusCode) {
        case 200:
          return Card.fromJson(response.body);
        default:
          throw Exception(jsonDecode(response.body)['message']);
      }
    });
  }

  Future<Card> add(Card card) async {
    return await http
        .post(Uri.parse('$baseUrl$endpoint'),
            headers: {
              'content-type': 'application/json',
              'x-auth-token': applicationToken ?? '',
              'x-auth-public': Config.instance.clientPublicKey,
              'x-auth-private': Config.instance.clientPrivateKey,
            },
            body: jsonEncode({'card': card}))
        .then((response) async {
      switch (response.statusCode) {
        case 200:
          return Card.fromJson(response.body);
        default:
          throw Exception(jsonDecode(response.body)['message']);
      }
    });
  }
}
