import 'dart:convert';
import 'package:apollopay/src/api/config.dart';
import 'package:apollopay/src/api/entities/paginated_entity.dart';
import 'package:apollopay/src/api/entities/payment/customer_entity.dart';
import 'package:apollopay/src/api/services/client_http_service.dart';
import 'package:http/http.dart' as http;

class CustomersService extends ClientHttpService {
  CustomersService({
    required String baseUrl,
    String endpoint = "/pay/customers/",
  }) : super(baseUrl: baseUrl, endpoint: endpoint);
  Future<Customer> getCustomerByCharge(String charge) async {
    return await http
        .get(Uri.parse('$baseUrl${endpoint}charge/$charge'), headers: {
      'content-type': 'application/json',
      'x-auth-token': applicationToken ?? '',
      'x-auth-private': Config.instance.clientPrivateKey,
      'x-auth-public': Config.instance.clientPublicKey,
    }).then((response) async {
      switch (response.statusCode) {
        case 200:
          return Customer.fromJson(response.body);
        default:
          throw Exception("Failed to get customer.");
      }
    });
  }

  Future<Paginated<Customer>> getCustomers(
      {Map<String, dynamic>? queryParameters}) async {
    String queryParams = "?";
    if (queryParameters != null) {
      queryParameters.forEach((key, value) {
        queryParams += '$key=$value&';
      });
    }
    return await http.get(Uri.parse('$baseUrl$endpoint$queryParams'), headers: {
      'content-type': 'application/json',
      'x-auth-token': applicationToken ?? '',
      'x-auth-private': Config.instance.clientPrivateKey,
      'x-auth-public': Config.instance.clientPublicKey,
    }).then((response) async {
      switch (response.statusCode) {
        case 200:
          Map data = jsonDecode(response.body);
          List<Customer> customers = [];

          for (var i = 0; i < data['data'].length; i++) {
            customers.add(Customer.fromMap(data['data'][i]));
          }
          var paginated = Paginated<Customer>.fromJson(data.cast());
          paginated.data = customers;
          return paginated;
        default:
          throw Exception("Failed to get customers.");
      }
    });
  }

  Future<Customer?> getCustomer(String customerToken) async {
    return await http.get(
      Uri.parse('$baseUrl$endpoint$customerToken'),
      headers: {
        'content-type': 'application/json',
        'x-auth-token': applicationToken ?? '',
        'x-auth-private': Config.instance.clientPrivateKey,
        'x-auth-public': Config.instance.clientPublicKey,
      },
    ).then((response) async {
      switch (response.statusCode) {
        case 200:
          return Customer.fromJson(response.body);
        default:
          return null;
      }
    });
  }
}
