import 'dart:convert';
import 'package:apollopay/src/api/config.dart';
import 'package:apollopay/src/api/entities/payment/pricing/price_entity.dart';
import 'package:apollopay/src/api/services/client_http_service.dart';
import 'package:http/http.dart' as http;

class PricesService extends ClientHttpService {
  PricesService({
    required String baseUrl,
    String endpoint = "/pay/prices/",
  }) : super(baseUrl: baseUrl, endpoint: endpoint);

  Future<List<Price>> getPricesForProduct(String productToken) async {
    return await http
        .get(Uri.parse('$baseUrl${endpoint}product/$productToken'), headers: {
      'content-type': 'application/json',
      'x-auth-token': applicationToken ?? '',
      'x-auth-private': Config.instance.clientPrivateKey,
      'x-auth-public': Config.instance.clientPublicKey,
    }).then((response) async {
      switch (response.statusCode) {
        case 200:
          var data = jsonDecode(response.body);
          List<Price> prices = [];
          for (var i = 0; i < data.length; i++) {
            prices.add(Price.fromMap(data[i]));
          }
          return prices;
        default:
          throw Exception("Failed to get prices.");
      }
    });
  }

  Future<Price?> getPrice(String priceToken) async {
    return await http.get(
      Uri.parse('$baseUrl$endpoint$priceToken'),
      headers: {
        'content-type': 'application/json',
        'x-auth-token': applicationToken ?? '',
        'x-auth-private': Config.instance.clientPrivateKey,
        'x-auth-public': Config.instance.clientPublicKey,
      },
    ).then((response) async {
      switch (response.statusCode) {
        case 200:
          return Price.fromJson(response.body);
        default:
          return null;
      }
    });
  }

  Future<Price?> createPrice(Price price) async {
    return await http
        .post(Uri.parse('$baseUrl$endpoint'),
            headers: {
              'content-type': 'application/json',
              'x-auth-token': applicationToken ?? '',
              'x-auth-private': Config.instance.clientPrivateKey,
              'x-auth-public': Config.instance.clientPublicKey,
            },
            body: jsonEncode({"price": price.toMap()}))
        .then((response) async {
      switch (response.statusCode) {
        case 200:
          return Price.fromJson(response.body);
        default:
          return null;
      }
    });
  }

  Future<bool?> modifyPrice(Price price) async {
    return await http
        .patch(Uri.parse('$baseUrl$endpoint${price.token}'),
            headers: {
              'content-type': 'application/json',
              'x-auth-token': applicationToken ?? '',
              'x-auth-private': Config.instance.clientPrivateKey,
              'x-auth-public': Config.instance.clientPublicKey,
            },
            body: jsonEncode({"price": price.toMap()}))
        .then((response) async {
      switch (response.statusCode) {
        case 200:
          return true;
        default:
          return null;
      }
    });
  }

  Future<bool?> deletePrice(String priceToken) async {
    return await http
        .delete(Uri.parse('$baseUrl$endpoint$priceToken'), headers: {
      'content-type': 'application/json',
      'x-auth-token': applicationToken ?? '',
      'x-auth-private': Config.instance.clientPrivateKey,
      'x-auth-public': Config.instance.clientPublicKey,
    }).then((response) async {
      switch (response.statusCode) {
        case 200:
          return true;
        default:
          return null;
      }
    });
  }
}
