import 'dart:convert';
import 'package:apollopay/src/api/config.dart';
import 'package:apollopay/src/api/entities/paginated_entity.dart';
import 'package:apollopay/src/api/entities/payment/refund_entity.dart';
import 'package:apollopay/src/api/services/client_http_service.dart';
import 'package:http/http.dart' as http;

class RefundsService extends ClientHttpService {
  RefundsService({
    required String baseUrl,
    String endpoint = "/pay/refunds/",
  }) : super(baseUrl: baseUrl, endpoint: endpoint);

  Future<Paginated<Refund>> getRefunds(
      {Map<String, dynamic>? queryParameters}) async {
    String queryParams = "?";

    if (queryParameters != null) {
      queryParameters.forEach((key, value) {
        queryParams += '$key=$value&';
      });
    }

    return await http.get(Uri.parse('$baseUrl$endpoint$queryParams'), headers: {
      'content-type': 'application/json',
      'x-auth-token': applicationToken ?? '',
      'x-auth-private': Config.instance.clientPrivateKey,
      'x-auth-public': Config.instance.clientPublicKey,
    }).then((response) async {
      switch (response.statusCode) {
        case 200:
          Map data = jsonDecode(response.body);
          List<Refund> refunds = [];

          for (var i = 0; i < data['data'].length; i++) {
            refunds.add(Refund.fromMap(data['data'][i]));
          }

          var paginated = Paginated<Refund>.fromJson(data.cast());

          paginated.data = refunds;

          return paginated;
        default:
          throw Exception("Failed to send currency.");
      }
    });
  }

  Future<Refund> getRefund(String refund) async {
    return await http.get(Uri.parse('$baseUrl$endpoint$refund'), headers: {
      'content-type': 'application/json',
      'x-auth-token': applicationToken ?? '',
      'x-auth-private': Config.instance.clientPrivateKey,
      'x-auth-public': Config.instance.clientPublicKey,
    }).then((response) async {
      switch (response.statusCode) {
        case 200:
          return Refund.fromJson(response.body);
        default:
          throw Exception("Failed to get Refund.");
      }
    });
  }

  Future<Refund> getRefundByCharge(String charge) async {
    return await http
        .get(Uri.parse('$baseUrl${endpoint}charge/$charge'), headers: {
      'content-type': 'application/json',
      'x-auth-token': applicationToken ?? '',
      'x-auth-private': Config.instance.clientPrivateKey,
      'x-auth-public': Config.instance.clientPublicKey,
    }).then((response) async {
      switch (response.statusCode) {
        case 200:
          return Refund.fromJson(response.body);
        default:
          throw Exception("Failed to get Refund.");
      }
    });
  }

  Future<Refund> refundCharge(String charge) async {
    return await http.post(Uri.parse('$baseUrl$endpoint$charge'), headers: {
      'content-type': 'application/json',
      'x-auth-token': applicationToken ?? '',
      'x-auth-private': Config.instance.clientPrivateKey,
      'x-auth-public': Config.instance.clientPublicKey,
    }).then((response) async {
      switch (response.statusCode) {
        case 200:
          return Refund.fromJson(response.body);
        default:
          throw Exception("Failed to get Refund.");
      }
    });
  }
}
