import 'dart:convert';
import 'package:apollopay/src/api/config.dart';
import 'package:apollopay/src/api/entities/payment/charge_entity.dart';
import 'package:apollopay/src/api/entities/payment/reports/sales_daily_report_entity.dart';
import 'package:apollopay/src/api/services/client_http_service.dart';
import 'package:http/http.dart' as http;

class ReportsService extends ClientHttpService {
  ReportsService({
    required String baseUrl,
    String endpoint = "/pay/reports/",
  }) : super(baseUrl: baseUrl, endpoint: endpoint);

  Future<dynamic> getSalesDailyReport(
      SalesDailyReport report, List<ChargeState> states) async {
    return await http
        .post(Uri.parse('$baseUrl${endpoint}sales/daily'),
            headers: {
              'content-type': 'application/json',
              'x-auth-token': applicationToken ?? '',
              'x-auth-private': Config.instance.clientPrivateKey,
              'x-auth-public': Config.instance.clientPublicKey,
            },
            body: jsonEncode({
              "report": report,
              "states": states.map((e) => e.index).toList()
            }))
        .then((response) async {
      switch (response.statusCode) {
        case 200:
          return jsonDecode(response.body);
        default:
          throw Exception("Failed to add event.");
      }
    });
  }
}
