import 'dart:convert';
import 'package:apollopay/src/api/config.dart';
import 'package:apollopay/src/api/entities/paginated_entity.dart';
import 'package:apollopay/src/api/entities/payment/charge_entity.dart';
import 'package:apollopay/src/api/services/client_http_service.dart';
import 'package:http/http.dart' as http;

class TransactionsService extends ClientHttpService {
  TransactionsService({
    required String baseUrl,
    String endpoint = "/pay/charges/",
  }) : super(baseUrl: baseUrl, endpoint: endpoint);

  Future<Paginated<Charge>> getCharges(
      {Map<String, dynamic>? queryParameters}) async {
    String queryParams = "?";
    if (queryParameters != null) {
      queryParameters.forEach((key, value) {
        queryParams += '$key=$value&';
      });
    }
    return await http.get(Uri.parse('$baseUrl$endpoint$queryParams'), headers: {
      'content-type': 'application/json',
      'x-auth-token': applicationToken ?? '',
      'x-auth-private': Config.instance.clientPrivateKey,
      'x-auth-public': Config.instance.clientPublicKey,
    }).then((response) async {
      switch (response.statusCode) {
        case 200:
          Map data = jsonDecode(response.body);
          List<Charge> charges = [];

          for (var i = 0; i < data['data'].length; i++) {
            charges.add(Charge.fromMap(data['data'][i]));
          }

          var paginated = Paginated<Charge>.fromJson(data.cast());

          paginated.data = charges;

          return paginated;
        default:
          throw Exception("Failed to send currency.");
      }
    });
  }

  Future<Charge> getCharge(String charge) async {
    return await http.get(Uri.parse('$baseUrl$endpoint$charge'), headers: {
      'content-type': 'application/json',
      'x-auth-token': applicationToken ?? '',
      'x-auth-private': Config.instance.clientPrivateKey,
      'x-auth-public': Config.instance.clientPublicKey,
    }).then((response) async {
      switch (response.statusCode) {
        case 200:
          return Charge.fromJson(response.body);
        default:
          throw Exception("Failed to get Charge.");
      }
    });
  }

  Future<Charge> processChargeForInvoice(
      String invoiceToken, Charge charge) async {
    return await http
        .post(Uri.parse('$baseUrl${endpoint}invoice/$invoiceToken'),
            headers: {
              'content-type': 'application/json',
              'x-auth-token': applicationToken ?? '',
              'x-auth-private': Config.instance.clientPrivateKey,
              'x-auth-public': Config.instance.clientPublicKey,
            },
            body: jsonEncode({'charge': charge}))
        .then((response) async {
      switch (response.statusCode) {
        case 200:
          return Charge.fromJson(response.body);
        default:
          throw Exception("Failed to get Charge.");
      }
    });
  }
}
