import 'dart:convert';
import 'package:apollopay/src/api/config.dart';
import 'package:apollopay/src/api/entities/crypto/currency_entity.dart';
import 'package:apollopay/src/api/entities/crypto/wallet_entity.dart';
import 'package:apollopay/src/api/services/client_http_service.dart';
import 'package:http/http.dart' as http;

class WalletsService extends ClientHttpService {
  WalletsService({
    required String baseUrl,
    String endpoint = "/crypto/wallets/",
  }) : super(baseUrl: baseUrl, endpoint: endpoint);

  Future<List<Wallet>> getWallets() async {
    return await http.get(Uri.parse('$baseUrl$endpoint'), headers: {
      'content-type': 'application/json',
      'x-auth-token': applicationToken ?? '',
      'x-auth-private': Config.instance.clientPrivateKey,
      'x-auth-public': Config.instance.clientPublicKey,
    }).then((response) async {
      switch (response.statusCode) {
        case 200:
          var data = jsonDecode(response.body);
          List<Wallet> users = [];
          for (var i = 0; i < data.length; i++) {
            users.add(Wallet.fromMap(data[i]));
          }
          return users;
        default:
          throw Exception("Failed to get wallets.");
      }
    });
  }

  Future<Wallet> getWallet(String walletId) async {
    return await http.get(Uri.parse('$baseUrl$endpoint$walletId'), headers: {
      'content-type': 'application/json',
      'x-auth-token': applicationToken ?? '',
      'x-auth-private': Config.instance.clientPrivateKey,
      'x-auth-public': Config.instance.clientPublicKey,
    }).then((response) async {
      switch (response.statusCode) {
        case 200:
          return Wallet.fromJson(response.body);
        default:
          throw Exception("Failed to get wallet.");
      }
    });
  }

  Future<String> createWallet(Currency currency) async {
    return await http
        .post(Uri.parse('$baseUrl$endpoint${currency.index}'), headers: {
      'content-type': 'application/json',
      'x-auth-token': applicationToken ?? '',
      'x-auth-private': Config.instance.clientPrivateKey,
      'x-auth-public': Config.instance.clientPublicKey,
    }).then((response) async {
      switch (response.statusCode) {
        case 200:
          return response.body;
        default:
          throw Exception("Failed to get wallets.");
      }
    });
  }
}
